@@ -23,7 +23,7 @@ glmc_mat4x2_zero(mat4x2 mat);
 
 CGLM_EXPORT
 void
-glmc_mat4x2_make(float * __restrict src, mat4x2 dest);
+glmc_mat4x2_make(const float * __restrict src, mat4x2 dest);
 
 CGLM_EXPORT
 void
