@@ -23,7 +23,7 @@ glmc_mat4x3_zero(mat4x3 mat);
 
 CGLM_EXPORT
 void
-glmc_mat4x3_make(float * __restrict src, mat4x3 dest);
+glmc_mat4x3_make(const float * __restrict src, mat4x3 dest);
 
 CGLM_EXPORT
 void
