@@ -22,7 +22,7 @@ glmc_mat2x3_zero(mat2x3 mat) {
 
 CGLM_EXPORT
 void
-glmc_mat2x3_make(float * __restrict src, mat2x3 dest) {
+glmc_mat2x3_make(const float * __restrict src, mat2x3 dest) {
   glm_mat2x3_make(src, dest);
 }
 
