@@ -22,7 +22,7 @@ glmc_mat2x4_zero(mat2x4 mat) {
 
 CGLM_EXPORT
 void
-glmc_mat2x4_make(float * __restrict src, mat2x4 dest) {
+glmc_mat2x4_make(const float * __restrict src, mat2x4 dest) {
   glm_mat2x4_make(src, dest);
 }
 
