@@ -22,7 +22,7 @@ glmc_mat3x2_zero(mat3x2 mat) {
 
 CGLM_EXPORT
 void
-glmc_mat3x2_make(float * __restrict src, mat3x2 dest) {
+glmc_mat3x2_make(const float * __restrict src, mat3x2 dest) {
   glm_mat3x2_make(src, dest);
 }
 
