@@ -392,7 +392,12 @@ TEST_DECLARE(euler)
 
 /* ray */
 TEST_DECLARE(glm_ray_triangle)
+TEST_DECLARE(glm_ray_sphere)
+TEST_DECLARE(glm_ray_at)
+
 TEST_DECLARE(glmc_ray_triangle)
+TEST_DECLARE(glmc_ray_sphere)
+TEST_DECLARE(glmc_ray_at)
 
 /* quat */
 TEST_DECLARE(MACRO_GLM_QUAT_IDENTITY_INIT)
@@ -530,6 +535,8 @@ TEST_DECLARE(glm_vec2_lerp)
 TEST_DECLARE(glm_vec2_complex_mul)
 TEST_DECLARE(glm_vec2_complex_div)
 TEST_DECLARE(glm_vec2_make)
+TEST_DECLARE(glm_vec2_reflect)
+TEST_DECLARE(glm_vec2_refract)
 
 TEST_DECLARE(glmc_vec2)
 TEST_DECLARE(glmc_vec2_copy)
@@ -576,6 +583,8 @@ TEST_DECLARE(glmc_vec2_lerp)
 TEST_DECLARE(glmc_vec2_complex_mul)
 TEST_DECLARE(glmc_vec2_complex_div)
 TEST_DECLARE(glmc_vec2_make)
+TEST_DECLARE(glmc_vec2_reflect)
+TEST_DECLARE(glmc_vec2_refract)
 
 /* vec3 */
 TEST_DECLARE(MACRO_GLM_VEC3_ONE_INIT)
@@ -678,6 +687,9 @@ TEST_DECLARE(glm_vec3_fract)
 TEST_DECLARE(glm_vec3_hadd)
 TEST_DECLARE(glm_vec3_sqrt)
 TEST_DECLARE(glm_vec3_make)
+TEST_DECLARE(glm_vec3_faceforward)
+TEST_DECLARE(glm_vec3_reflect)
+TEST_DECLARE(glm_vec3_refract)
 
 TEST_DECLARE(glmc_vec3)
 TEST_DECLARE(glmc_vec3_copy)
@@ -754,6 +766,9 @@ TEST_DECLARE(glmc_vec3_fract)
 TEST_DECLARE(glmc_vec3_hadd)
 TEST_DECLARE(glmc_vec3_sqrt)
 TEST_DECLARE(glmc_vec3_make)
+TEST_DECLARE(glmc_vec3_faceforward)
+TEST_DECLARE(glmc_vec3_reflect)
+TEST_DECLARE(glmc_vec3_refract)
 
 /* vec4 */
 TEST_DECLARE(MACRO_GLM_VEC4_ONE_INIT)
@@ -842,6 +857,8 @@ TEST_DECLARE(glm_vec4_fract)
 TEST_DECLARE(glm_vec4_hadd)
 TEST_DECLARE(glm_vec4_sqrt)
 TEST_DECLARE(glm_vec4_make)
+TEST_DECLARE(glm_vec4_reflect)
+TEST_DECLARE(glm_vec4_refract)
 
 TEST_DECLARE(glmc_vec4)
 TEST_DECLARE(glmc_vec4_copy3)
@@ -914,18 +931,25 @@ TEST_DECLARE(glmc_vec4_fract)
 TEST_DECLARE(glmc_vec4_hadd)
 TEST_DECLARE(glmc_vec4_sqrt)
 TEST_DECLARE(glmc_vec4_make)
+TEST_DECLARE(glmc_vec4_reflect)
+TEST_DECLARE(glmc_vec4_refract)
 
 /* ivec2 */
 TEST_DECLARE(glm_ivec2)
 TEST_DECLARE(glm_ivec2_copy)
 TEST_DECLARE(glm_ivec2_zero)
 TEST_DECLARE(glm_ivec2_one)
+TEST_DECLARE(glm_ivec2_dot)
+TEST_DECLARE(glm_ivec2_cross)
 TEST_DECLARE(glm_ivec2_add)
 TEST_DECLARE(glm_ivec2_adds)
 TEST_DECLARE(glm_ivec2_sub)
 TEST_DECLARE(glm_ivec2_subs)
 TEST_DECLARE(glm_ivec2_mul)
 TEST_DECLARE(glm_ivec2_scale)
+TEST_DECLARE(glm_ivec2_div)
+TEST_DECLARE(glm_ivec2_divs)
+TEST_DECLARE(glm_ivec2_mod)
 TEST_DECLARE(glm_ivec2_addadd)
 TEST_DECLARE(glm_ivec2_addadds)
 TEST_DECLARE(glm_ivec2_subadd)
@@ -956,12 +980,17 @@ TEST_DECLARE(glmc_ivec2)
 TEST_DECLARE(glmc_ivec2_copy)
 TEST_DECLARE(glmc_ivec2_zero)
 TEST_DECLARE(glmc_ivec2_one)
+TEST_DECLARE(glmc_ivec2_dot)
+TEST_DECLARE(glmc_ivec2_cross)
 TEST_DECLARE(glmc_ivec2_add)
 TEST_DECLARE(glmc_ivec2_adds)
 TEST_DECLARE(glmc_ivec2_sub)
 TEST_DECLARE(glmc_ivec2_subs)
 TEST_DECLARE(glmc_ivec2_mul)
 TEST_DECLARE(glmc_ivec2_scale)
+TEST_DECLARE(glmc_ivec2_div)
+TEST_DECLARE(glmc_ivec2_divs)
+TEST_DECLARE(glmc_ivec2_mod)
 TEST_DECLARE(glmc_ivec2_addadd)
 TEST_DECLARE(glmc_ivec2_addadds)
 TEST_DECLARE(glmc_ivec2_subadd)
@@ -993,12 +1022,18 @@ TEST_DECLARE(glm_ivec3)
 TEST_DECLARE(glm_ivec3_copy)
 TEST_DECLARE(glm_ivec3_zero)
 TEST_DECLARE(glm_ivec3_one)
+TEST_DECLARE(glm_ivec3_dot)
+TEST_DECLARE(glm_ivec3_norm2)
+TEST_DECLARE(glm_ivec3_norm)
 TEST_DECLARE(glm_ivec3_add)
 TEST_DECLARE(glm_ivec3_adds)
 TEST_DECLARE(glm_ivec3_sub)
 TEST_DECLARE(glm_ivec3_subs)
 TEST_DECLARE(glm_ivec3_mul)
 TEST_DECLARE(glm_ivec3_scale)
+TEST_DECLARE(glm_ivec3_div)
+TEST_DECLARE(glm_ivec3_divs)
+TEST_DECLARE(glm_ivec3_mod)
 TEST_DECLARE(glm_ivec3_addadd)
 TEST_DECLARE(glm_ivec3_addadds)
 TEST_DECLARE(glm_ivec3_subadd)
@@ -1028,12 +1063,18 @@ TEST_DECLARE(glmc_ivec3)
 TEST_DECLARE(glmc_ivec3_copy)
 TEST_DECLARE(glmc_ivec3_zero)
 TEST_DECLARE(glmc_ivec3_one)
+TEST_DECLARE(glmc_ivec3_dot)
+TEST_DECLARE(glmc_ivec3_norm2)
+TEST_DECLARE(glmc_ivec3_norm)
 TEST_DECLARE(glmc_ivec3_add)
 TEST_DECLARE(glmc_ivec3_adds)
 TEST_DECLARE(glmc_ivec3_sub)
 TEST_DECLARE(glmc_ivec3_subs)
 TEST_DECLARE(glmc_ivec3_mul)
 TEST_DECLARE(glmc_ivec3_scale)
+TEST_DECLARE(glmc_ivec3_div)
+TEST_DECLARE(glmc_ivec3_divs)
+TEST_DECLARE(glmc_ivec3_mod)
 TEST_DECLARE(glmc_ivec3_addadd)
 TEST_DECLARE(glmc_ivec3_addadds)
 TEST_DECLARE(glmc_ivec3_subadd)
@@ -1526,10 +1567,15 @@ TEST_LIST {
   TEST_ENTRY(glmc_euler_zyx_quat_lh)
 
   TEST_ENTRY(euler)
-  
+
   /* ray */
   TEST_ENTRY(glm_ray_triangle)
+  TEST_ENTRY(glm_ray_sphere)
+  TEST_ENTRY(glm_ray_at)
+
   TEST_ENTRY(glmc_ray_triangle)
+  TEST_ENTRY(glmc_ray_sphere)
+  TEST_ENTRY(glmc_ray_at)
 
   /* quat */
   TEST_ENTRY(MACRO_GLM_QUAT_IDENTITY_INIT)
@@ -1666,6 +1712,8 @@ TEST_LIST {
   TEST_ENTRY(glm_vec2_complex_mul)
   TEST_ENTRY(glm_vec2_complex_div)
   TEST_ENTRY(glm_vec2_make)
+  TEST_ENTRY(glm_vec2_reflect)
+  TEST_ENTRY(glm_vec2_refract)
 
   TEST_ENTRY(glmc_vec2)
   TEST_ENTRY(glmc_vec2_copy)
@@ -1712,6 +1760,8 @@ TEST_LIST {
   TEST_ENTRY(glmc_vec2_complex_mul)
   TEST_ENTRY(glmc_vec2_complex_div)
   TEST_ENTRY(glmc_vec2_make)
+  TEST_ENTRY(glmc_vec2_reflect)
+  TEST_ENTRY(glmc_vec2_refract)
 
   /* vec3 */
   TEST_ENTRY(MACRO_GLM_VEC3_ONE_INIT)
@@ -1813,6 +1863,9 @@ TEST_LIST {
   TEST_ENTRY(glm_vec3_hadd)
   TEST_ENTRY(glm_vec3_sqrt)
   TEST_ENTRY(glm_vec3_make)
+  TEST_ENTRY(glm_vec3_faceforward)
+  TEST_ENTRY(glm_vec3_reflect)
+  TEST_ENTRY(glm_vec3_refract)
 
   TEST_ENTRY(glmc_vec3)
   TEST_ENTRY(glmc_vec3_copy)
@@ -1889,6 +1942,9 @@ TEST_LIST {
   TEST_ENTRY(glmc_vec3_hadd)
   TEST_ENTRY(glmc_vec3_sqrt)
   TEST_ENTRY(glmc_vec3_make)
+  TEST_ENTRY(glmc_vec3_faceforward)
+  TEST_ENTRY(glmc_vec3_reflect)
+  TEST_ENTRY(glmc_vec3_refract)
 
   /* vec4 */
   TEST_ENTRY(MACRO_GLM_VEC4_ONE_INIT)
@@ -1977,6 +2033,8 @@ TEST_LIST {
   TEST_ENTRY(glm_vec4_hadd)
   TEST_ENTRY(glm_vec4_sqrt)
   TEST_ENTRY(glm_vec4_make)
+  TEST_ENTRY(glm_vec4_reflect)
+  TEST_ENTRY(glm_vec4_refract)
 
   TEST_ENTRY(glmc_vec4)
   TEST_ENTRY(glmc_vec4_copy3)
@@ -2049,18 +2107,25 @@ TEST_LIST {
   TEST_ENTRY(glmc_vec4_hadd)
   TEST_ENTRY(glmc_vec4_sqrt)
   TEST_ENTRY(glmc_vec4_make)
+  TEST_ENTRY(glmc_vec4_reflect)
+  TEST_ENTRY(glmc_vec4_refract)
 
   /* ivec2 */
   TEST_ENTRY(glm_ivec2)
   TEST_ENTRY(glm_ivec2_copy)
   TEST_ENTRY(glm_ivec2_zero)
   TEST_ENTRY(glm_ivec2_one)
+  TEST_ENTRY(glm_ivec2_dot)
+  TEST_ENTRY(glm_ivec2_cross)
   TEST_ENTRY(glm_ivec2_add)
   TEST_ENTRY(glm_ivec2_adds)
   TEST_ENTRY(glm_ivec2_sub)
   TEST_ENTRY(glm_ivec2_subs)
   TEST_ENTRY(glm_ivec2_mul)
   TEST_ENTRY(glm_ivec2_scale)
+  TEST_ENTRY(glm_ivec2_div)
+  TEST_ENTRY(glm_ivec2_divs)
+  TEST_ENTRY(glm_ivec2_mod)
   TEST_ENTRY(glm_ivec2_addadd)
   TEST_ENTRY(glm_ivec2_addadds)
   TEST_ENTRY(glm_ivec2_subadd)
@@ -2091,12 +2156,17 @@ TEST_LIST {
   TEST_ENTRY(glmc_ivec2_copy)
   TEST_ENTRY(glmc_ivec2_zero)
   TEST_ENTRY(glmc_ivec2_one)
+  TEST_ENTRY(glmc_ivec2_dot)
+  TEST_ENTRY(glmc_ivec2_cross)
   TEST_ENTRY(glmc_ivec2_add)
   TEST_ENTRY(glmc_ivec2_adds)
   TEST_ENTRY(glmc_ivec2_sub)
   TEST_ENTRY(glmc_ivec2_subs)
   TEST_ENTRY(glmc_ivec2_mul)
   TEST_ENTRY(glmc_ivec2_scale)
+  TEST_ENTRY(glmc_ivec2_div)
+  TEST_ENTRY(glmc_ivec2_divs)
+  TEST_ENTRY(glmc_ivec2_mod)
   TEST_ENTRY(glmc_ivec2_addadd)
   TEST_ENTRY(glmc_ivec2_addadds)
   TEST_ENTRY(glmc_ivec2_subadd)
@@ -2128,12 +2198,18 @@ TEST_LIST {
   TEST_ENTRY(glm_ivec3_copy)
   TEST_ENTRY(glm_ivec3_zero)
   TEST_ENTRY(glm_ivec3_one)
+  TEST_ENTRY(glm_ivec3_dot)
+  TEST_ENTRY(glm_ivec3_norm2)
+  TEST_ENTRY(glm_ivec3_norm)
   TEST_ENTRY(glm_ivec3_add)
   TEST_ENTRY(glm_ivec3_adds)
   TEST_ENTRY(glm_ivec3_sub)
   TEST_ENTRY(glm_ivec3_subs)
   TEST_ENTRY(glm_ivec3_mul)
   TEST_ENTRY(glm_ivec3_scale)
+  TEST_ENTRY(glm_ivec3_div)
+  TEST_ENTRY(glm_ivec3_divs)
+  TEST_ENTRY(glm_ivec3_mod)
   TEST_ENTRY(glm_ivec3_addadd)
   TEST_ENTRY(glm_ivec3_addadds)
   TEST_ENTRY(glm_ivec3_subadd)
@@ -2163,12 +2239,18 @@ TEST_LIST {
   TEST_ENTRY(glmc_ivec3_copy)
   TEST_ENTRY(glmc_ivec3_zero)
   TEST_ENTRY(glmc_ivec3_one)
+  TEST_ENTRY(glmc_ivec3_dot)
+  TEST_ENTRY(glmc_ivec3_norm2)
+  TEST_ENTRY(glmc_ivec3_norm)
   TEST_ENTRY(glmc_ivec3_add)
   TEST_ENTRY(glmc_ivec3_adds)
   TEST_ENTRY(glmc_ivec3_sub)
   TEST_ENTRY(glmc_ivec3_subs)
   TEST_ENTRY(glmc_ivec3_mul)
   TEST_ENTRY(glmc_ivec3_scale)
+  TEST_ENTRY(glmc_ivec3_div)
+  TEST_ENTRY(glmc_ivec3_divs)
+  TEST_ENTRY(glmc_ivec3_mod)
   TEST_ENTRY(glmc_ivec3_addadd)
   TEST_ENTRY(glmc_ivec3_addadds)
   TEST_ENTRY(glmc_ivec3_subadd)
